<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Teknisa\GER\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response\Error;

class Opl00200TipoOperacaoOperadorLogisticoController extends TeknisaCrud {
    protected $crudService;
	protected $dataSourceManager;
	protected $parameterBag;
	protected $environment;
    protected $dataSourceName;
    protected $parameterBagColumns = array();
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment) {
        parent::__construct($crudService, $parameterBag);
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $P_CDFILIAL = null;
        $P_OPERLOG = null;
        $P_CDOPERADOR = null;
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "TNMFILIAL") {
                if($condition["value"] == 'T') {
                    $P_CDFILIAL = 'T';
                }
            } else if($condition["columnName"] == "NMFILIALL") {
                if(is_array($condition["value"])) {
                    if(count($condition["value"]) != 0) {
                        $filialStr = '_';
                        foreach($condition["value"] as $filial) {
                            $filialStr .= $filial . '_';
                        }
                        $P_CDFILIAL = $filialStr;
                    } else {
                        $P_CDFILIAL = 'T';
                    }
                } else {
                    $P_CDFILIAL = 'T';
                }
            } else if($condition["columnName"] == "CDFILIOPELOG") {
                $P_OPERLOG = $condition["value"];
            }
            
        }
        
        $P_CDOPERADOR = $this->environment->getCdOperador();
        
        $filter = array(
            array(
                "columnName" => "P_CDFILIAL",
                "value" => $P_CDFILIAL,
                "operator" => "="
            ),
            array(
                "columnName" => "P_CDOPERADOR",
                "value" => $P_CDOPERADOR,
                "operator" => "="
            ),
            array(
                "columnName" => "P_OPERLOG",
                "value" => $P_OPERLOG,
                "operator" => "="
            )
        );
        
        $parameterBagColumns = array("P_CDFILIAL", "P_CDOPERADOR", "P_OPERLOG");
        
        $this->dataSourceName = "opl00200_get";
        
        $newFilterCriteria = new FilterCriteria($this->dataSourceName, $filter);
        $newRequest = new Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    
        parent::find($newRequest, $response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = "opl00200_crud";
        
        try {
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = "opl00200_crud";
        
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}