<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\OPL\Modules\Consolidacao\Services\Sup07200LancamentoDePedidosService;

class Sup07200LancamentoDePedidosController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $newCode;
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, Sup07200LancamentoDePedidosService $service, $environment, $newCode) {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
		$this->service      = $service;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
    }
    
    public function retornaFloat($valor) {
        if(gettype($valor) === 'string') {
	        $valor = str_replace('.', '', $valor);
            $valor = str_replace(',', '.', $valor);
	        $valor = (float) $valor;
	    }
	    return $valor;
	}
	
	public function getCentralCompras(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $centralDeCompras = $this->service->getCentralCompras();
            
            if(!$centralDeCompras) {
                throw new \Exception("CDFILCENTCMP_NOT_REGISTERED");
            }
            $response->addDataSet(new DataSet("data", $centralDeCompras));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function checkIgualdadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->service->checkIgualdadeItemPedidoOL($row);
            
            if(isset($result[0]["CONTADOR"])) {
                if($result[0]["CONTADOR"] > 0) {
                    throw new \Exception("ITEM_ALREADY_EXISTS_PEDIDO", 500);
                }
            } else {
                throw new \Exception("ERRO_QUERY", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function insereItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
            
            if($row["QTPROGENPEOL"] == 0) {
                throw new \Exception("QTPROGENPEOL_EQUALS_ZERO", 500);
            } else {
                $this->service->insereItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function excluiItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->excluiItemPedidoOL($row);
                $this->service->excluiItemPedidoOLItemSoli($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraDataEntregaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $row["OLD_DTPROGENPEOL"] = $row["DTPROGENPEOL"];
                $row["DTPROGENPEOL"] = $row["NEW_DTPROGENPEOL"];
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("DTPROGENPEOL_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraDataEntregaItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraMarcaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $row["CDMARCA"] = $row["NEW_CDMARCA"];
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("CDMARCA_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraMarcaItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraProdutoItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $row["CDPRODUTO"] = $row["NEW_CDPRODUTO"];
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["AJUSTE_UNIDADE"] = self::retornaFloat($row["AJUSTE_UNIDADE"]);
                $row["AJUSTE_QUANTIDADE"] = self::retornaFloat($row["AJUSTE_QUANTIDADE"]);
                
                $QTDAJUSTADA = round(($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"])) + 
                    (($row["AJUSTE_QUANTIDADE"] / 100) * ($row["QTPROGENPEOL"] + (($row["AJUSTE_UNIDADE"] / 100) * $row["QTPROGENPEOL"]))), 3);
                    
                $row["NEW_QTPROGENPEOL"] = $QTDAJUSTADA;
                    
                $result = $this->service->checkIgualdadeItemPedidoOL($row);
                
                if(isset($result[0]["CONTADOR"])) {
                    if($result[0]["CONTADOR"] > 0) {
                        throw new \Exception("CDPRODUTO_ALREADY_EXISTS_PEDIDO", 500);
                    } else {
                        $this->service->alteraProdutoItemPedidoOL($row);
                    }
                } else {
                    throw new \Exception("ERRO_QUERY", 500);
                }
            }
            
            if($aux == 0)
                $this->service->realizaCommit();
            else
                $this->service->realizaRollback();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function alteraQuantidadeItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["NEW_QTPROGENPEOL"] = self::retornaFloat($row["NEW_QTPROGENPEOL"]);
                $row["OLD_QTPROGENPEOL"] = self::retornaFloat($row["OLD_QTPROGENPEOL"]);
                
                $this->service->alteraQuantidadeItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function incluiItemPedidoLote(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $this->service->inclusaoDePedidoParaLote($rows[0]);
            
            foreach($rows as $row) {
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function incluiItemPedidoLoteDiversos(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->inclusaoDePedidoParaLote($row);
                $this->service->incluiItemPedidoLote($row);
                $this->service->ajusteItemPedidoLote($row);
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function inclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $aux = 0;
            
            foreach($rows as $row) {
                $QUANTESTOQ = null;
                $QUANTRESRV = null;
                $QUANTITEMP = null;
                $CDOLFILIAL = $this->service->verificaOLQueAtendeFilial($row);
                $QUANTESTOQ = $this->service->posicaoEstoqueItemOL($CDOLFILIAL, $row);
                $QUANTRESRV = $this->service->verificaReservaDemaisPedidos($row);
                $QUANTITEMP = self::retornaFloat($row["QTPROGENPEOL"]);
                
                $QTRESERVESTOL = (($QUANTESTOQ - $QUANTRESRV) - $QUANTITEMP);
                
                if($QTRESERVESTOL >= 0) {
                    $this->service->incluiReservaItemPedidoOL($row, $QTRESERVESTOL);
                } else {
                    $aux++;
                }
            }
            
            if($aux == 0) {
                $this->service->realizaCommit();
            } else {
                $this->service->realizaRollback();
                throw new \Exception("QTD_INDISPONIVEL_RESERVA", 500);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
    
    public function exclusaoReservaItemPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->excluiReservaItemPedidoLote($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function baixaManualItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["QTORIGPRODPED"] = self::retornaFloat($row["QTORIGPRODPED"]);
                $this->service->atualizaSituacaoItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function checkOperBlockUnblockPer(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->service->checkOperBlockUnblockPer($row);
            
            if(isset($result[0]["CDOPERADOR"])) {
                $response->addDataSet(new DataSet("CDOPERADOR", $result[0]));
            } else {
                
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function desbloqueioBloqueioItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $this->service->desbloqueioBloqueioItemPedidoOL($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function validacaoSenhaED(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $dataSourceName = "operador";
        $conditions = array(array("columnName" => "CDOPERADOR", "operator" => "=", "value" => $row["CDOPERADOR"]));
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        if (md5($row["SENHA"]) != $dataSet[0]["CDSENHAOPERWEB"]) {
            $response->setError(new Error("INVALID_PASSWORD", 1));
        }
    }
    
    public function processoEntregaDiretaItemPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $adqParametros = $this->service->getAdqParametros($row);
                
                if($adqParametros["IDGERAITEMSOLI"] == 'N') {
                    $this->service->acmSolPontoAPontoItemPedidoED($row);
                } else {
                    $adqItemSoliEd = $this->service->getAdqItemSoliEd($row);
                    
                    if(is_null($adqItemSoliEd)) {
                        $adqPesqItemSoli = $this->service->getAdqPesqItemSoli($row);
                        
                        if(!isset($adqPesqItemSoli["NRSOLICMP"]) || is_null($adqPesqItemSoli["NRSOLICMP"])) {
                            $this->service->acmInsSolicitaItemPedidoED($row);
                            $this->service->acmInsSolPontoaPontoItemPedidoED($row, $adqParametros, null);
                        } else {
                            $this->service->acmInsSolPontoaPontoItemPedidoED($row, $adqParametros, $adqPesqItemSoli);
                            $this->service->acmInsMovopSolItemPedidoED($row, $adqPesqItemSoli);
                        }
                    } else {
                        $this->service->acmSolPontoaPonto2ItemPedidoED($row, $adqItemSoliEd);
                    }
                }
                
                $this->service->excluiItemPedidoOL($row);
                $this->service->excluiItemPedidoOLItemSoli($row);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function carregaDadosPedido(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $result = $this->service->carregaDadosPedido($row);
            
            $response->addDataSet(new DataSet("opl07200_dados_pedido", $result));
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function salvaDadosPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function baixaManualPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["NEW_IDBAIXENPEOL"] = $row["NEW_IDBAIXEXOL"];
                $item["NEW_DTBAIXTMPEOL"] = $row["NEW_DTBAIXTMPEOL"];
                $this->service->atualizaSitItemViaPedido($item);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function cancelaEncerraPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function emAbertoPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $this->service->salvaDadosPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["NEW_IDBAIXENPEOL"] = $row["NEW_IDBAIXEXOL"];
                $this->service->atualizaSitItemViaPedido($item);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function exclusaoPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            //$this->service->exclusaoItensPedido($row);
            
            $itens = $this->service->selecionaItensDoPedido($row);
            
            foreach($itens as $item) {
                $item["CDFILIOPELOG"] = $row["CDFILIOPELOG"];
                
                $this->service->excluiItemPedidoOL($item);
                $this->service->excluiItemPedidoOLItemSoli($item);
            }
            
            $this->service->exclusaoPedido($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function inclusaoPedido(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            $rowDados = $row["DADOS"];
            $rowProdutos = $row["PRODUTOS"];
            
            $NEWNRPEDEXOL = $this->service->inclusaoPedido($rowDados);
            
            foreach($rowProdutos as $rowProd) {
                $rowProd["NRPEDEXOL"] = $NEWNRPEDEXOL;
                $rowProd["CDLOTENVITOL"] = isset($rowDados["CDLOTENVITOL"]) ? $rowDados["CDLOTENVITOL"] : null;
                $rowProd["CDFILLANC"] = $rowDados["CDFILLANC"];
                $rowProd["CDFILDEST"] = $rowDados["CDFILDEST"];
                $rowProd["QTPROGENPEOL"] = self::retornaFloat($rowProd["QTPROGENPEOL"]);
                $rowProd["CDFILIOPELOG"] = $rowDados["CDFILIOPELOG"];
                
                if($rowProd["QTPROGENPEOL"] > 0)
                    $this->service->insereItemPedidoOL($rowProd);
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function exportarPedidoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $fatorPreco = $this->service->getFatorPreco();
            
            $row["CDFATPRECO"] = isset($fatorPreco["CDFATPRECO"]) ? $fatorPreco["CDFATPRECO"] : null;
            
            $retornoProcedure = $this->service->procedureGeraPedOLExpedicao($row);
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
	    }
    }
    
    public function importacaoSolicitacaoOL(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $row = $request->getRow();
            $novaLista = $this->service->procedureGeraNovoCodigo("LISTA");
            
            $this->service->insereNovaLista($novaLista, $row);
            
            $adqValidacoes = $this->service->adqValidacoes($novaLista, $row);
            
            $this->service->deleteNovaLista($novaLista);
            
            if(count($adqValidacoes) > 0) {
                $retornoProcedure = $this->service->incluiPedidoOLProcedure($row);
                
                if($retornoProcedure["P_ROWSAFFECT"] == 'S') {
                    $this->service->realizaCommit();
                } else {
                    throw new \Exception("ERROR_APUR", 500);
                }
            } else {
                throw new \Exception("APUR_NOT_RESULTS", 500);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
	        $this->service->realizaRollback();
        }
    }
    
    public function alteraQuantidadeItemPedidoOLInline(DTO\Request\Row $request, DTO\Response $response) {
        $this->service->beginTransaction();
        
        try {
            $rows = $request->getRow();
            
            foreach($rows as $row) {
                $row["QTPROGENPEOL"] = self::retornaFloat($row["QTPROGENPEOL"]);
                $row["NEW_QTPROGENPEOL"] = self::retornaFloat($row["NEW_QTPROGENPEOL"]);
                $row["OLD_QTPROGENPEOL"] = self::retornaFloat($row["OLD_QTPROGENPEOL"]);
                
                if($row["NEW_QTPROGENPEOL"] == 0) {
                    $this->service->excluiItemPedidoOL($row);
                    $this->service->excluiItemPedidoOLItemSoli($row);
                } else {
                    $this->service->alteraQuantidadeItemPedidoOL($row);
                }
            }
            
            $this->service->realizaCommit();
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
            $this->service->realizaRollback();
	    }
    }
}